import { POST_PATH } from "@blog/config";
import { readdir } from "node:fs/promises";
import type { Post } from "@blog/models/Post";
import * as yamlFront from "yaml-front-matter";
import { Marked } from "marked";
import { markedHighlight } from "marked-highlight";
import hljs from "highlight.js";

export function getMarkedInstance(): Marked {
  return new Marked(
    markedHighlight({
      langPrefix: 'hljs language-',
      highlight(code, lang, info) {
        const language = hljs.getLanguage(lang) ? lang : 'plaintext';
        return hljs.highlight(code, { language }).value;
      }
    })
  )
}

export async function readPostFile(path: string): Promise<Post> {
  const file = Bun.file(path);
  const fileContent = await file.text();
  const parsedData = yamlFront.loadFront(fileContent);
  const marked: Marked = getMarkedInstance();
  const postHtml = await marked.parse(parsedData.__content);

  let slug = parsedData.slug;

  if (!slug) {
    // get the last object in the path, remove the last three characters,
    // presuming they are `.md`
    slug = path.split("/").pop()?.slice(0, -3);
  }

  return {
    meta: {
      title: parsedData.title,
      description: parsedData.description,
      date: new Date(parsedData.date),
      draft: parsedData?.draft || false,
      tags: parsedData.tags,
      slug: slug,
    },
    content: parsedData.__content,
    html: postHtml,
  };
}

export class PostFileService {
  private posts: Map<string, Post>;

  public constructor(posts: Map<string, Post>) {
    this.posts = posts;
  }

  public static async create() {
    const posts = new Map<string, Post>();
    const postFiles: string[] = await readdir(POST_PATH);

    for (const postFile of postFiles) {
      const post = await readPostFile(POST_PATH + "/" + postFile);
      const key = post.meta?.slug || postFile.slice(0, -3);
      posts.set(key, post);
    }

    return new PostFileService(posts);
  }

  public getPosts(): Map<string, Post> {
    return this.posts;
  }

  public getPostsSortedByDate(desc: boolean = true): Post[] {
    return Array.from(this.posts.values())
      .sort((p1: Post, p2: Post) => {
        if (desc) {
          return p1.meta.date.getTime() - p2.meta.date.getTime();
        }
      
        return p2.meta.date.getTime() - p1.meta.date.getTime();
      });
  }

  public getPublishedPosts(): Post[] {
    return this.getPostsSortedByDate()
      .filter((p: Post) => p.meta.draft == false);
  }

  public getPost(slug: string): Post {
    const post = this.posts.get(slug);

    if (!post) {
      throw Error("Post does not exist.");
    }

    return post;
  }

  public getPostHtml(slug: string): string {
    const html = this.getPost(slug).html;

    if (!html) {
      throw Error("No HTML for this post yet.");
    }

    return html;
  }

  public getPostRawContent(slug: string): string {
    return this.getPost(slug).content;
  }
}
